module Rsb
  module Gol
    class Game
      CLEAR_SCREEN_CHAR = "\e[H\e[2J"

      attr_reader :universe, :stop_running
      attr_accessor :seed

      def initialize(options = {})
        random_seed   = options[:random_seed] == false ? false : true
        rows          = options[:rows] || 40
        cols          = options[:cols] || 80
        grid          = Rsb::Gol::Grid.new(rows, cols, random_seed)
        @universe     = options[:universe] || Rsb::Gol::Universe.new(grid: grid)
        @stop_running = false

        trap('INT') do
          puts "\ngame of life will now stop running\n"
          @stop_running = true
        end
      end

      def clear_screen
        puts CLEAR_SCREEN_CHAR
      end

      def status_line(data)
        "@@@ generation clock: #{data[:clock]} " +
          "deaths: #{data[:deaths]} births: #{data[:births]}"
      end

      def start(iterations = 1000, alive='0', dead=' ', step=3)
        clear_screen
        iterations.times do
          exit if stop_running
          data   = universe.tick!
          abort('empty universe') if data[:empty_universe]
          status = status_line(data)
          output = universe.visualize(alive, dead)

          clear_screen
          puts "\n\n#{output}\n\n #{status}\n"
          sleep(1.0/ step)
        end
      end
    end
  end
end
