require 'matrix'

module Rsb
  module Gol
    class Universe
      DEFAULT_ROWS    = 50
      DEFAULT_COLUMNS = 50
      attr_reader :clock

      def initialize(options = {})
        @grid = if options[:grid]
                  options[:grid]
                elsif options[:cols] && options[:rows]
                  Rsb::Gol::Grid.new(options[:rows], options[:cols])
                else
                  Rsb::Gol::Grid.new(DEFAULT_ROWS, DEFAULT_COLUMNS)
                end
        @clock = 0
      end

      def evolution
        deaths = []
        births = []

        @grid.cell_neighbors do |state, x, y, neighbors|
          if state == 1
            deaths << [x, y] if neighbors.size < 2 || neighbors.size > 3
          end

          if state == 0
            births << [x, y] if neighbors.size == 3
          end
        end

        { deaths: deaths, births: births }
      end

      def populate(population)
        population[:deaths].each {|(x, y)| @grid.kill(x, y)}
        population[:births].each {|(x, y)| @grid.spawn(x, y)}
        {
          deaths: population[:deaths].size,
          births: population[:births].size
        }
      end

      def seed(coords)
        coords.each do |x, y|
          @grid.spawn(x,y)
        end
      end

      def visualize(alive = 'o', dead = ' ')
        @grid.visualize(alive, dead)
      end

      def tick!
        @clock += 1
        stats = populate(evolution())
        {
          empty_universe: stats[:deaths] == 0 && stats[:births] == 0,
          deaths: stats[:deaths],
          births: stats[:births],
          clock: clock
        }
      end
    end
  end
end
